@@ -3,7 +3,6 @@
 # Copyright (c) Mark Gillard <mark.gillard@outlook.com.au>
 # See https://github.com/marzer/poxy/blob/master/LICENSE for the full license text.
 # SPDX-License-Identifier: MIT
-
 """
 Low-level helper functions and useful bits.
 """
@@ -12,24 +11,25 @@ import sys
 import re
 import io
 import logging
-from typing import Tuple
+import requests
+from typing import Tuple, Union
 from pathlib import Path
-from io import StringIO
 from misk import *
-
-# __all__ = []
+from . import dirs
 
 #=======================================================================================================================
 # FUNCTIONS
 #=======================================================================================================================
 
-def regex_or(patterns, pattern_prefix = '', pattern_suffix = '', flags=0):
+
+
+def regex_or(patterns, pattern_prefix='', pattern_suffix='', flags=0):
 	patterns = [str(r) for r in patterns if r is not None and r]
 	patterns.sort()
 	pattern = ''
 	if patterns:
 		pattern = '(?:(?:' + ')|(?:'.join(patterns) + '))'
-	patterns = re.compile(pattern_prefix + pattern + pattern_suffix, flags=flags)
+	patterns = re.compile(rf'{pattern_prefix}{pattern}{pattern_suffix}', flags=flags)
 	return patterns
 
 
@@ -56,23 +56,23 @@ def combine_dicts(x, y):
 
 
 
-_is_uri_regex = re.compile(r'^[a-zA-Z][a-zA-Z0-9_]*://.+$')
+RX_IS_URI = re.compile(r'^[a-zA-Z][a-zA-Z0-9_]*://.+$')
+
+
+
 def is_uri(s):
-	global _is_uri_regex
-	return _is_uri_regex.fullmatch(str(s)) is not None
+	global RX_IS_URI
+	return RX_IS_URI.fullmatch(str(s)) is not None
 
 
 
-_lib_version = None
 def lib_version() -> Tuple[int, int, int]:
-	global _lib_version
-	if _lib_version is None:
-		data_dir = Path(Path(__file__).resolve().parent, r'data')
-		with open(Path(data_dir, 'version.txt'), encoding='utf-8') as file:
-			_lib_version = [int(v.strip()) for v in file.read().strip().split('.')]
-			assert len(_lib_version) == 3
-			_lib_version = tuple(_lib_version)
-	return _lib_version
+	if not hasattr(lib_version, "val"):
+		with open(Path(dirs.DATA, 'version.txt'), encoding='utf-8') as file:
+			lib_version.val = [int(v.strip()) for v in file.read().strip().split('.')]
+			assert len(lib_version.val) == 3
+			lib_version.val = tuple(lib_version.val)
+	return lib_version.val
 
 
 
@@ -85,10 +85,34 @@ def filter_filenames(files, include, exclude):
 
 
 
+DOWNLOAD_HEADERS = {r'User-Agent': r'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:104.0) Gecko/20100101 Firefox/104.0'}
+
+
+
+def download_text(uri: str, timeout=10, encoding='utf-8') -> str:
+	assert uri is not None
+	global DOWNLOAD_HEADERS
+	response = requests.get(str(uri), headers=DOWNLOAD_HEADERS, timeout=timeout)
+	if encoding is not None:
+		response.encoding = encoding
+	return response.text
+
+
+
+def download_binary(uri: str, timeout=10) -> bytes:
+	assert uri is not None
+	global DOWNLOAD_HEADERS
+	response = requests.get(str(uri), headers=DOWNLOAD_HEADERS, timeout=timeout)
+	return response.content
+
+
+
 #=======================================================================================================================
 # REGEX REPLACER
 #=======================================================================================================================
 
+
+
 class RegexReplacer(object):
 
 	def __substitute(self, m):
@@ -119,6 +143,8 @@ class RegexReplacer(object):
 # CppTree
 #=======================================================================================================================
 
+
+
 class CppTree(object):
 
 	NAMESPACES = 1
@@ -127,7 +153,7 @@ class CppTree(object):
 
 	class Node(object):
 
-		def __init__(self, val, parent, type_ = 0):
+		def __init__(self, val, parent, type_=0):
 			assert val.find(r'::') == -1
 			assert type_ in (0, CppTree.NAMESPACES, CppTree.TYPES, CppTree.ENUM_VALUES)
 			self.value = val
@@ -136,7 +162,7 @@ class CppTree(object):
 			self.mask = type_
 			self.children = {}
 
-		def add(self, val, type_ = 0):
+		def add(self, val, type_=0):
 			assert val.find(r'::') == -1
 			assert type_ in (0, CppTree.NAMESPACES, CppTree.TYPES, CppTree.ENUM_VALUES)
 			child = None
@@ -167,7 +193,7 @@ class CppTree(object):
 
 			grouped = len(matchers) > 1
 			matchers = r'|'.join(matchers)
-			if not self.value and not self.parent: # root
+			if not self.value and not self.parent:  # root
 				return matchers
 			matchers = (r'(?:' if grouped else '') + matchers + (r')' if grouped else '')
 
@@ -176,8 +202,6 @@ class CppTree(object):
 			else:
 				return rf'{self.value}::{matchers}'
 
-
-
 	def __init__(self):
 		self.root = CppTree.Node('', None)
 
@@ -207,6 +231,8 @@ class CppTree(object):
 # Custom exceptions
 #=======================================================================================================================
 
+
+
 class Error(Exception):
 	"""Base class for other exceptions."""
 
